{% extends "base/page.html" %}

{% block content %}
<section>
<h2>No algorithms installed!</h2>
<p>
    This can happen for a variety of reasons. Check the logs for more information.
</p>
</section>
{% endblock %}
